-- This file contains variables and functions related to GUI.
if not gui then gui = {} end

-- Possible main popup content types. Used in show_main_popup function.
gui.main_popup_content_type =
{
	enable_creative_mode = 1,
	permanent_disable_creative_mode = -1,
	
	enable_more_cheats = 2,
	disable_more_cheats = -2
}

-- Shows a generic yes/no popup with elements of given names for the given player.
-- @param	warning_label_name		Optional. Name of the warning label. No warning label will be shown if it is not provided.
-- @param	warning_label_caption	Optional. Caption for the warning label. No warning label will be shown if it is not provided.
function gui.show_yes_no_popup(player, frame_name, frame_caption, btn_flow_name, yes_btn_name, no_btn_name, warning_label_name, warning_label_caption)
	local center = player.gui.center
	center.add({type = "frame", name = frame_name, direction = "vertical", caption = {frame_caption}})
	local popup = center[frame_name]
	
	if warning_label_caption then
		popup.add({type = "label", name = warning_label_name, caption = {warning_label_caption}})
		popup[warning_label_name].style.font_color = {r=0.98, g=0.66, b=0.22}
	end
	
	popup.add({type = "flow", name = btn_flow_name, direction = "horizontal"})
	local buttons_flow = popup[btn_flow_name]
	
	if yes_btn_name then
		buttons_flow.add({type = "button", name = yes_btn_name, caption = {creative_mode_defines.names.gui_captions.yes}})
	end
	if no_btn_name then
		buttons_flow.add({type = "button", name = no_btn_name, caption = {creative_mode_defines.names.gui_captions.no}})
	end
end

-- Shows the more cheats popup for enabling or disabling the cheats.
function gui.show_more_cheats_popup(player, enable)
	-- Prepare the captions.
	local popup_caption
	local label_caption
	local cheat_mode_caption
	local loaders_caption
	local tech_caption
	local always_day_caption
	local reach_distance_caption
	local mining_speed_caption
	local running_speed_caption
	if enable then
		popup_caption = {"gui.creative-mode_cheats-to-enable"}
		label_caption = {"gui.creative-mode_enable-more-cheats"}
		cheat_mode_caption = {"gui.creative-mode_enable-cheat-mode"}
		loaders_caption = {"gui.creative-mode_enable-loaders"}
		tech_caption = {"gui.creative-mode_research-all-technologies"}
		always_day_caption = {"gui.creative-mode_make-all-surfaces-always-day"}
		reach_distance_caption = {"gui.creative-mode_increase-character-reach"}
		mining_speed_caption = {"gui.creative-mode_increase-character-mining-speed"}
		running_speed_caption = {"gui.creative-mode_increase-character-running-speed"}
	else
		popup_caption = {"gui.creative-mode_cheats-to-disable"}
		label_caption = {"gui.creative-mode_disable-more-cheats"}
		cheat_mode_caption = {"gui.creative-mode_disable-cheat-mode"}
		loaders_caption = {"gui.creative-mode_disable-loaders"}
		tech_caption = {"gui.creative-mode_reset-all-technologies"}
		always_day_caption = {"gui.creative-mode_bring-back-night-to-surfaces"}
		reach_distance_caption = {"gui.creative-mode_reset-character-reach"}
		mining_speed_caption = {"gui.creative-mode_reset-character-mining-speed"}
		running_speed_caption = {"gui.creative-mode_reset-character-running-speed"}
	end

	local center = player.gui.center
	center.add({type = "frame", name = creative_mode_defines.names.gui.popup, direction = "vertical", caption = popup_caption})
	local popup = center[creative_mode_defines.names.gui.popup]
	
	popup.add({type = "table", name = creative_mode_defines.names.gui.cheats_table, colspan  = 2})
	local cheats_table = popup[creative_mode_defines.names.gui.cheats_table]
		
	-- Label, and the select all button.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_label, caption = label_caption})
	cheats_table.add({type = "button", name = creative_mode_defines.names.gui.cheats_select_all_btn, caption = {"gui.creative-mode_all"}})
	-- Cheat mode.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_cheat_mode_label, caption = cheat_mode_caption, tooltip = {"gui.creative-mode_cheat-mode-tooltip"}})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_cheat_mode_checkbox, state = true})
	-- Loaders.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_loaders_label, caption = loaders_caption, tooltip = {"gui.creative-mode_loaders-tooltip"}})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_loaders_checkbox, state = true})
	-- Technologies.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_technologies_label, caption = tech_caption})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_technologies_checkbox, state = true})
	-- Always day.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_always_day_label, caption = always_day_caption})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_always_day_checkbox, state = true})
	-- Character reach.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_character_reach_label, caption = reach_distance_caption, tooltip = {"gui.creative-mode_no-effect-in-god-mode-tooltip"}})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_character_reach_checkbox, state = true})
	-- Character mining speed.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_character_mining_speed_label, caption = mining_speed_caption, tooltip = {"gui.creative-mode_no-effect-in-god-mode-tooltip"}})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox, state = true})
	-- Character running speed.
	cheats_table.add({type = "label", name = creative_mode_defines.names.gui.cheats_character_running_speed_label, caption = running_speed_caption, tooltip = {"gui.creative-mode_no-effect-in-god-mode-tooltip"}})
	cheats_table.add({type = "checkbox", name = creative_mode_defines.names.gui.cheats_character_running_speed_checkbox, state = true})
	
	-- Styles.
	cheats_table[creative_mode_defines.names.gui.cheats_label].style.bottom_padding = 20
	cheats_table[creative_mode_defines.names.gui.cheats_select_all_btn].style.font = "default-small-bold"
	cheats_table[creative_mode_defines.names.gui.cheats_character_running_speed_label].style.bottom_padding = 20
	cheats_table[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox].style.bottom_padding = 20
	
	-- Done button.
	if enable then
		popup.add({type = "button", name = creative_mode_defines.names.gui.cheats_enable_done, caption = {"gui.creative-mode_done"}})
	else
		popup.add({type = "button", name = creative_mode_defines.names.gui.cheats_disable_done, caption = {"gui.creative-mode_done"}})
	end
end

-- Shows the main popup for the given player.
function gui.show_main_popup(player, main_popup_content_type)
	if main_popup_content_type == gui.main_popup_content_type.enable_creative_mode then
		-- Enable Creative Mode?
		global.creative_mode.has_asked_for_enable = true
		gui.show_yes_no_popup(player,
			creative_mode_defines.names.gui.popup,
			"gui.creative-mode_ask-enable-creative-mode",
			creative_mode_defines.names.gui.popup_buttons_flow,
			creative_mode_defines.names.gui.enable_creative_mode_yes,
			creative_mode_defines.names.gui.enable_creative_mode_no,
			nil)
	elseif main_popup_content_type == gui.main_popup_content_type.permanent_disable_creative_mode then
		-- Permanently disable Creative Mode?
		gui.show_yes_no_popup(player,
			creative_mode_defines.names.gui.popup,
			"gui.creative-mode_ask-permanently-disable-creative-mode",
			creative_mode_defines.names.gui.popup_buttons_flow,
			creative_mode_defines.names.gui.permanent_disable_creative_mode_yes,
			creative_mode_defines.names.gui.permanent_disable_creative_mode_no,
			creative_mode_defines.names.gui.popup_label,
			"gui.creative-mode_permanently-disable-creative-mode-info")
	
	elseif main_popup_content_type == gui.main_popup_content_type.enable_more_cheats then
		-- Enable more cheats?
		gui.show_more_cheats_popup(player, true)
	elseif main_popup_content_type == gui.main_popup_content_type.disable_more_cheats then
		-- Disable more cheats?
		gui.show_more_cheats_popup(player, false)
	end
end

-- Destroys the main popup for the given player.
function gui.destroy_main_popup(player)
	player.gui.center[creative_mode_defines.names.gui.popup].destroy()
end

-- Selects or desleects the checkboxes of cheats according to their states on the cheats popup.
function gui.select_all_cheats(popup)
	local checkbox_container = popup[creative_mode_defines.names.gui.cheats_table]
	local checkboxes = {}
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_cheat_mode_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_loaders_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_technologies_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_always_day_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_character_reach_checkbox])
	table.insert(checkboxes, checkbox_container[creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox])
	
	-- Find whether we should select or deselect them.
	local should_select = false
	for _, cb in ipairs(checkboxes) do
		if not cb.state then
			should_select = true
			break
		end
	end
	
	-- Action.
	for _, cb in ipairs(checkboxes) do
		cb.state = should_select
	end
end

-- Callback of the on_gui_click event, which is invoked when a GUI element is clicked by any player.
function gui.on_gui_click(event)
	local player = game.players[event.element.player_index]
	local popup = player.gui.center[creative_mode_defines.names.gui.popup]
	if event.element.name == creative_mode_defines.names.gui.enable_creative_mode_yes then
		-- Enable Creative Mode? Yes
		gui.destroy_main_popup(player)
		cheats.enable_or_disable_creative_mode(true, false, false, player)
	elseif event.element.name == creative_mode_defines.names.gui.enable_creative_mode_no then
		-- Enable Creative Mode? No
		gui.destroy_main_popup(player)
		if global.creative_mode.config.misc.show_hint_after_creative_mode_denied then
			player.print({"message.creative-mode_creative-mode_denied", command_enable})
		end
		
	elseif event.element.name == creative_mode_defines.names.gui.permanent_disable_creative_mode_yes then
		-- Permanently disable Creative Mode? Yes
		gui.destroy_main_popup(player)
		cheats.enable_or_disable_creative_mode(false, true, false, player)
	elseif event.element.name == creative_mode_defines.names.gui.permanent_disable_creative_mode_no then
		-- Permanently disable Creative Mode? No
		gui.destroy_main_popup(player)
		
	---------------------------------------------------------------------------------------------------
		
	elseif event.element.name == creative_mode_defines.names.gui.cheats_select_all_btn then
		-- Select all cheats.
		gui.select_all_cheats(popup)
		
	elseif event.element.name == creative_mode_defines.names.gui.cheats_enable_done then
		-- Enable cheats done.
		local checkbox_container = popup[creative_mode_defines.names.gui.cheats_table]
		if checkbox_container[creative_mode_defines.names.gui.cheats_cheat_mode_checkbox].state then
			cheats.enable_or_disable_cheat_mode(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_loaders_checkbox].state then
			cheats.enable_or_disable_loaders(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_technologies_checkbox].state then
			cheats.research_or_reset_all_technologies(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_always_day_checkbox].state then
			cheats.enable_or_disable_always_day(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_reach_checkbox].state then
			cheats.increase_or_reset_character_reach(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox].state then
			cheats.increase_or_reset_character_mining_speed(true)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox].state then
			cheats.increase_or_reset_character_running_speed(true)
		end
		gui.destroy_main_popup(player)
		
	elseif event.element.name == creative_mode_defines.names.gui.cheats_disable_done then
		-- Disable cheats done.
		local checkbox_container = popup[creative_mode_defines.names.gui.cheats_table]
		if checkbox_container[creative_mode_defines.names.gui.cheats_cheat_mode_checkbox].state then
			cheats.enable_or_disable_cheat_mode(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_loaders_checkbox].state then
			cheats.enable_or_disable_loaders(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_technologies_checkbox].state then
			cheats.research_or_reset_all_technologies(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_always_day_checkbox].state then
			cheats.enable_or_disable_always_day(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_reach_checkbox].state then
			cheats.increase_or_reset_character_reach(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_mining_speed_checkbox].state then
			cheats.increase_or_reset_character_mining_speed(false)
		end
		if checkbox_container[creative_mode_defines.names.gui.cheats_character_running_speed_checkbox].state then
			cheats.increase_or_reset_character_running_speed(false)
		end
		gui.destroy_main_popup(player)
	end
end