-- This file contains variables or functions that are not specific to any mod and can be used by other mods.
if not util then util = {} end

-- The distance between items on transport belt.
-- Last update: Factorio 0.13.3
transport_belt_item_distance = 0.28125

-- Returns whether the given sorted table contain the given specific value.
function util.has_value_sorted(tab, val)
	for _, v in ipairs(tab) do
		if v == val then
			return true
		end
	end
	return false
end

-- Prints the given message to all players.
function util.print_to_all_players(message)
	for _, player in pairs(game.players) do
		player.print(message)
	end
end

-- Returns the bounding box of the tile at the given position.
-- @param	position	It should contain {x: float, y: float}
function util.get_tile_bb(position)
	return
	{
		{math.floor(position.x), math.floor(position.y)},
		{math.ceil(position.x), math.ceil(position.y)}
	}
end

-- Returns whether the given entity has at least one inventory.
function util.has_inventory(entity)
	for _, inv in pairs(defines.inventory) do
		if entity.get_inventory(inv) ~= nil then
			return true
		end
	end
	return false
end

-- Returns whether the given entity has at least one fluidbox.
function util.has_fluidbox(entity)
	if not entity.fluidbox then
		return false
	end
	return #entity.fluidbox > 0
end

-- Returns whether the given inserter entity is enabled according to its circuit network state as well as its logistic network state.
function util.is_inserter_enabled(inserter)
	local control = inserter.get_control_behavior()
	-- Does it have control behaviour? (Not connected = no control?)
	if control and control.valid then
		-- Check logistic network.
		-- Is logistic network connected?
		if control.connect_to_logistic_network then
			-- Has logistic condition set?
			if control.logistic_condition then
				-- Condition fulfilled?
				if not control.logistic_condition.fulfilled then
					return false
				end
			else
				-- Connected but no condition, hence not OK.
				return false
			end			
		end
	
		-- Check circuit network.
		-- Is connected by wire?
		if control.get_circuit_network(defines.wire_type.red, defines.circuit_connector_id.inserter) or control.get_circuit_network(defines.wire_type.green, defines.circuit_connector_id.inserter) then
			-- Is operation mode enable/disable?
			if control.circuit_mode_of_operation == defines.control_behavior.inserter.circuit_mode_of_operation.enable_disable then
				-- Has the circuit condition set?
				if control.circuit_condition then
					-- Condition fulfilled?
					if not control.circuit_condition.fulfilled then
						return false
					end
				else
					-- Connected but no condition, hence not OK.
					return false
				end
			end
		end
	end
	
	-- No control? Because not connected to network?
	return true
end